VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAPIFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Uses Windows API for file access
'
'Author
'  Name .....: Josip Medved
'  E-mail ...: jmedved@vip.hr
'  URL ......: http://web.vip.hr/jmedved.vip/qeyes/

'Information
'  Additional : -
'  Requires ..: -
'  Has help ..: No
'  Has readme : No
'  Has sample : Yes
'  Created ...: 2002/07/24 (yyyy/mm/dd)
'  Last update: 2002/09/09 (yyyy/mm/dd)
'  Notes .....: -

'If you use this file in your program it would be nice to mention
'where you got it from, but you are not obligated to do so.


Option Explicit

Private Const FILE_BEGIN = 0

Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, ByVal lpSecurityAttributes As Long, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, ByVal hTemplateFile As Long) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Private Declare Function DeleteFile Lib "kernel32" Alias "DeleteFileA" (ByVal lpFileName As String) As Long
Private Declare Function FlushFileBuffers Lib "kernel32" (ByVal hFile As Long) As Long
Private Declare Function GetFileSize Lib "kernel32" (ByVal hFile As Long, lpFileSizeHigh As Long) As Long
Private Declare Function GetLastError Lib "kernel32" () As Long
Private Declare Function ReadFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, lpNumberOfBytesRead As Long, ByVal lpOverlapped As Long) As Long
Private Declare Function SetEndOfFile Lib "kernel32" (ByVal hFile As Long) As Long
Private Declare Function SetFilePointer Lib "kernel32" (ByVal hFile As Long, ByVal lDistanceToMove As Long, lpDistanceToMoveHigh As Long, ByVal dwMoveMethod As Long) As Long
Private Declare Function WriteFile Lib "kernel32" (ByVal hFile As Long, ByRef lpBuffer As Byte, ByVal nNumberOfBytesToWrite As Long, ByRef lpNumberOfBytesWritten As Long, ByVal lpOverlapped As Long) As Long


Public Enum qeAPIFileLastError
  qAPIFileLastErrorNone = 0
  qAPIFileLastErrorFileNotFound = 53
  qAPIFileLastErrorBadFileMode = 54
  qAPIFileLastErrorFileClosed = 54
  qAPIFileLastErrorFileOpened = 55
  qAPIFileLastErrorBadRecordLength = 59
  qAPIFileLastErrorBadRecordNumber = 63
  qAPIFileLastErrorToManyFiles = 67
  qAPIFileLastErrorPermissionDenied = 70
End Enum

Public Enum qeAPIFileAccess
  qAPIFileAccessRead = &H80000000
  qAPIFileAccessWrite = &H40000000
  qAPIFileAccessReadWrite = &H40000000 Or &H80000000
End Enum

Public Enum qeAPIFileShare
  qAPIFileShareRead = 1
  qAPIFileShareWrite = 2
  qAPIFileShareReadWrite = 1 Or 2
End Enum

Public Enum qeAPIFileCreate
  qAPIFileCreateNew = 1
  qAPIFileCreateAlways = 2 'if file exists it creates it anyway
  qAPIFileOpenExisting = 3
  qAPIFileOpenAlways = 4
  qAPIFileTruncateExisting = 5
End Enum

Public Enum qeAPIFileAttributes
  qAPIFileAttributesArchive = &H20
  qAPIFileAttributesHidden = &H2
  qAPIFileAttributesNormal = &H80
  qAPIFileAttributesReadOnly = &H1
  qAPIFileAttributesSystem = &H4
  qAPIFileAttributesTemporary = &H100
End Enum

Public Enum qeAPIFileFlags
  qAPIFileFlagsNone = 0
  qAPIFileFlagsWriteThrough = &H80000000
  qAPIFileFlagsOverlapped = &H40000000
  qAPIFileFlagsNoBuffering = &H20000000
  qAPIFileFlagsRandomAccess = &H10000000
  qAPIFileFlagsSequentialScan = &H8000000
  qAPIFileFlagsDeleteOnClose = &H4000000
  qAPIFileFlagsBackupSemantics = &H2000000
  qAPIFileFlagsPosixSemantics = &H1000000
End Enum

Private xLastError As qeAPIFileLastError
Private xFileName As String
Private xFileHandle As Long

Public Sub FileOpen(ByRef FileName As String, Optional ByVal FileAccess As qeAPIFileAccess = qAPIFileAccessReadWrite, Optional FileShare As qeAPIFileShare = qAPIFileShareReadWrite, Optional ByVal FileCreate As qeAPIFileCreate = qAPIFileOpenAlways, Optional ByVal FileAttributes As qeAPIFileAttributes = qAPIFileAttributesNormal, Optional ByVal FileFlags As qeAPIFileFlags = qAPIFileFlagsNone)
  'Opens file
  '  FileName [in] - name of file to open
  '  FileAccess [in] - desired access
  '  FileShare [in] - desired share
  '  FileCreate [in] - creation flags
  '  FileAttributes [in] - desired file attributes
  '  FileFlags [in] - desired file flags
  xLastError = qAPIFileLastErrorNone
  If xFileHandle <> 0 Then xLastError = qAPIFileLastErrorFileOpened: Exit Sub

  Dim hFile As Long
  hFile = CreateFile(FileName, FileAccess, FileShare, 0&, FileCreate, FileAttributes Or FileFlags, 0&)

  Select Case hFile
    Case 0
      xLastError = GetLastError()
    Case -1
      xLastError = qAPIFileLastErrorBadFileMode
    Case Else
      xFileName = FileName
      xFileHandle = hFile
  End Select
End Sub

Public Sub FileClose(Optional ByVal FileDelete As Boolean = False)
  'Closes file
  '  FileDelete [in] - is file to be deleted after close
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Sub

  Dim tmpRet As Long
  CloseHandle xFileHandle
  If tmpRet = 0 Then xLastError = GetLastError

  If FileDelete = True Then
    tmpRet = DeleteFile(xFileName)
    If tmpRet = 0 Then xLastError = GetLastError
  End If
  
  xFileName = ""
  xFileHandle = 0
End Sub

Public Sub FileFlushBuffers()
  'Flushes buffered contest of file to disk
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Sub

  Dim tmpRet As Long
  tmpRet = FlushFileBuffers(xFileHandle)
  If tmpRet = 0 Then xLastError = GetLastError
End Sub

Public Sub FilePut(ByVal Offset As Long, ByRef Data() As Byte, Optional ByRef DataCount As Long = 0)
  'Writes to file
  '  Offset [in] - file offset (zero-based)
  '  Data() [in] - array of bytes to write
  '  DataCount [out] - count of bytes written
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Sub
  
  If Offset < 0 Then xLastError = qAPIFileLastErrorBadRecordNumber: Exit Sub

  On Error Resume Next
  Dim tmpCount As Long
  tmpCount = UBound(Data()) + 1

  If tmpCount > 0 Then
    Dim tmpRet As Long
    tmpRet = SetFilePointer(xFileHandle, Offset, 0, FILE_BEGIN)
    tmpRet = WriteFile(xFileHandle, Data(0), tmpCount, DataCount, 0&)
    If tmpRet = 0 Then xLastError = GetLastError
  End If
End Sub

Public Sub FileGet(ByVal Offset As Long, ByRef Data() As Byte, Optional ByRef DataCount As Long = 0)
  'Writes to file
  '  Offset [in] - file offset (zero-based)
  '  Data() [out] - array of bytes to read into
  '  DataCount [in,out] - count of bytes to read or read
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Sub

  If Offset < 0 Then xLastError = qAPIFileLastErrorBadRecordNumber: Exit Sub

  If DataCount > 0 Then
    ReDim Data(0 To DataCount - 1)
  End If

  On Error Resume Next
  Dim tmpCount As Long
  tmpCount = UBound(Data()) + 1

  If tmpCount > 0 Then
    Dim tmpRet As Long
    tmpRet = SetFilePointer(xFileHandle, Offset, 0, FILE_BEGIN)
    tmpRet = ReadFile(xFileHandle, Data(0), tmpCount, DataCount, 0&)
    If tmpRet = 0 Then xLastError = GetLastError
  End If
  If DataCount <> tmpCount Then
    If DataCount > 0 Then
      ReDim Data(0 To DataCount - 1)
    Else
      Erase Data()
    End If
  End If
End Sub

Public Property Get FileLength() As Long
  'Gets current filelength
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Property

  FileLength = GetFileSize(xFileHandle, 0&)
End Property

Public Property Let FileLength(ByVal Offset As Long)
  'Sets new file length
  '  Offset [in] -count of bytes to leave
  xLastError = qAPIFileLastErrorNone
  If xFileHandle = 0 Then xLastError = qAPIFileLastErrorFileClosed: Exit Property

  If Offset < 0 Then xLastError = qAPIFileLastErrorBadRecordNumber: Exit Property

  Dim tmpRet As Long
  tmpRet = SetFilePointer(xFileHandle, Offset, 0, FILE_BEGIN)
  tmpRet = SetEndOfFile(xFileHandle)
  If tmpRet = 0 Then xLastError = GetLastError
End Property


Public Property Get LastError() As qeAPIFileLastError
  'Last error
  LastError = xLastError
End Property


Private Sub Class_Initialize()
  '
End Sub

Private Sub Class_Terminate()
  FileClose
End Sub
