VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCRC32"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'CRC32
'
'Author
'  Name .....: Josip Medved
'  E-mail ...: jmedved@vip.hr
'  URL ......: http://web.vip.hr/jmedved.vip/qeyes/

'Information
'  Requires ..: -
'  Additional : -
'  Has help ..: No
'  Has readme : No
'  Has sample : No
'  Created ...: 2002/04/12 (yyyy/mm/dd)
'  Last update: 2002/09/12 (yyyy/mm/dd)
'  Notes .....: -

'If you use this file in your program it would be nice to mention
'where you got it from, but you are not obligated to do so.


Option Explicit

Private Declare Sub RtlMoveMemoryByte2Long Lib "kernel32.dll" Alias "RtlMoveMemory" (Destination As Long, Source As Byte, ByVal Length As Long)

Private CRC32LookupTable(0 To 255) As Long
Private CRC32Digest As Long

Public Sub Init(Optional ByVal Reserved As Long = &HEDB88320, Optional ByVal StartingDigest As Long = &HFFFFFFFF)
  'Initializes Lookup table and misc.
  '  Reserved [in] - reserved for future use
  '  StartingDigest [in] - from which number does CRCing start
  If Reserved = &HEDB88320 Then
    CRC32LookupTable(0) = &H0: CRC32LookupTable(1) = &H77073096: CRC32LookupTable(2) = &HEE0E612C: CRC32LookupTable(3) = &H990951BA: CRC32LookupTable(4) = &H76DC419: CRC32LookupTable(5) = &H706AF48F: CRC32LookupTable(6) = &HE963A535: CRC32LookupTable(7) = &H9E6495A3
    CRC32LookupTable(8) = &HEDB8832: CRC32LookupTable(9) = &H79DCB8A4: CRC32LookupTable(10) = &HE0D5E91E: CRC32LookupTable(11) = &H97D2D988: CRC32LookupTable(12) = &H9B64C2B: CRC32LookupTable(13) = &H7EB17CBD: CRC32LookupTable(14) = &HE7B82D07: CRC32LookupTable(15) = &H90BF1D91
    CRC32LookupTable(16) = &H1DB71064: CRC32LookupTable(17) = &H6AB020F2: CRC32LookupTable(18) = &HF3B97148: CRC32LookupTable(19) = &H84BE41DE: CRC32LookupTable(20) = &H1ADAD47D: CRC32LookupTable(21) = &H6DDDE4EB: CRC32LookupTable(22) = &HF4D4B551: CRC32LookupTable(23) = &H83D385C7
    CRC32LookupTable(24) = &H136C9856: CRC32LookupTable(25) = &H646BA8C0: CRC32LookupTable(26) = &HFD62F97A: CRC32LookupTable(27) = &H8A65C9EC: CRC32LookupTable(28) = &H14015C4F: CRC32LookupTable(29) = &H63066CD9: CRC32LookupTable(30) = &HFA0F3D63: CRC32LookupTable(31) = &H8D080DF5
    CRC32LookupTable(32) = &H3B6E20C8: CRC32LookupTable(33) = &H4C69105E: CRC32LookupTable(34) = &HD56041E4: CRC32LookupTable(35) = &HA2677172: CRC32LookupTable(36) = &H3C03E4D1: CRC32LookupTable(37) = &H4B04D447: CRC32LookupTable(38) = &HD20D85FD: CRC32LookupTable(39) = &HA50AB56B
    CRC32LookupTable(40) = &H35B5A8FA: CRC32LookupTable(41) = &H42B2986C: CRC32LookupTable(42) = &HDBBBC9D6: CRC32LookupTable(43) = &HACBCF940: CRC32LookupTable(44) = &H32D86CE3: CRC32LookupTable(45) = &H45DF5C75: CRC32LookupTable(46) = &HDCD60DCF: CRC32LookupTable(47) = &HABD13D59
    CRC32LookupTable(48) = &H26D930AC: CRC32LookupTable(49) = &H51DE003A: CRC32LookupTable(50) = &HC8D75180: CRC32LookupTable(51) = &HBFD06116: CRC32LookupTable(52) = &H21B4F4B5: CRC32LookupTable(53) = &H56B3C423: CRC32LookupTable(54) = &HCFBA9599: CRC32LookupTable(55) = &HB8BDA50F
    CRC32LookupTable(56) = &H2802B89E: CRC32LookupTable(57) = &H5F058808: CRC32LookupTable(58) = &HC60CD9B2: CRC32LookupTable(59) = &HB10BE924: CRC32LookupTable(60) = &H2F6F7C87: CRC32LookupTable(61) = &H58684C11: CRC32LookupTable(62) = &HC1611DAB: CRC32LookupTable(63) = &HB6662D3D
    CRC32LookupTable(64) = &H76DC4190: CRC32LookupTable(65) = &H1DB7106: CRC32LookupTable(66) = &H98D220BC: CRC32LookupTable(67) = &HEFD5102A: CRC32LookupTable(68) = &H71B18589: CRC32LookupTable(69) = &H6B6B51F: CRC32LookupTable(70) = &H9FBFE4A5: CRC32LookupTable(71) = &HE8B8D433
    CRC32LookupTable(72) = &H7807C9A2: CRC32LookupTable(73) = &HF00F934: CRC32LookupTable(74) = &H9609A88E: CRC32LookupTable(75) = &HE10E9818: CRC32LookupTable(76) = &H7F6A0DBB: CRC32LookupTable(77) = &H86D3D2D: CRC32LookupTable(78) = &H91646C97: CRC32LookupTable(79) = &HE6635C01
    CRC32LookupTable(80) = &H6B6B51F4: CRC32LookupTable(81) = &H1C6C6162: CRC32LookupTable(82) = &H856530D8: CRC32LookupTable(83) = &HF262004E: CRC32LookupTable(84) = &H6C0695ED: CRC32LookupTable(85) = &H1B01A57B: CRC32LookupTable(86) = &H8208F4C1: CRC32LookupTable(87) = &HF50FC457
    CRC32LookupTable(88) = &H65B0D9C6: CRC32LookupTable(89) = &H12B7E950: CRC32LookupTable(90) = &H8BBEB8EA: CRC32LookupTable(91) = &HFCB9887C: CRC32LookupTable(92) = &H62DD1DDF: CRC32LookupTable(93) = &H15DA2D49: CRC32LookupTable(94) = &H8CD37CF3: CRC32LookupTable(95) = &HFBD44C65
    CRC32LookupTable(96) = &H4DB26158: CRC32LookupTable(97) = &H3AB551CE: CRC32LookupTable(98) = &HA3BC0074: CRC32LookupTable(99) = &HD4BB30E2: CRC32LookupTable(100) = &H4ADFA541: CRC32LookupTable(101) = &H3DD895D7: CRC32LookupTable(102) = &HA4D1C46D: CRC32LookupTable(103) = &HD3D6F4FB
    CRC32LookupTable(104) = &H4369E96A: CRC32LookupTable(105) = &H346ED9FC: CRC32LookupTable(106) = &HAD678846: CRC32LookupTable(107) = &HDA60B8D0: CRC32LookupTable(108) = &H44042D73: CRC32LookupTable(109) = &H33031DE5: CRC32LookupTable(110) = &HAA0A4C5F: CRC32LookupTable(111) = &HDD0D7CC9
    CRC32LookupTable(112) = &H5005713C: CRC32LookupTable(113) = &H270241AA: CRC32LookupTable(114) = &HBE0B1010: CRC32LookupTable(115) = &HC90C2086: CRC32LookupTable(116) = &H5768B525: CRC32LookupTable(117) = &H206F85B3: CRC32LookupTable(118) = &HB966D409: CRC32LookupTable(119) = &HCE61E49F
    CRC32LookupTable(120) = &H5EDEF90E: CRC32LookupTable(121) = &H29D9C998: CRC32LookupTable(122) = &HB0D09822: CRC32LookupTable(123) = &HC7D7A8B4: CRC32LookupTable(124) = &H59B33D17: CRC32LookupTable(125) = &H2EB40D81: CRC32LookupTable(126) = &HB7BD5C3B: CRC32LookupTable(127) = &HC0BA6CAD
    CRC32LookupTable(128) = &HEDB88320: CRC32LookupTable(129) = &H9ABFB3B6: CRC32LookupTable(130) = &H3B6E20C: CRC32LookupTable(131) = &H74B1D29A: CRC32LookupTable(132) = &HEAD54739: CRC32LookupTable(133) = &H9DD277AF: CRC32LookupTable(134) = &H4DB2615: CRC32LookupTable(135) = &H73DC1683
    CRC32LookupTable(136) = &HE3630B12: CRC32LookupTable(137) = &H94643B84: CRC32LookupTable(138) = &HD6D6A3E: CRC32LookupTable(139) = &H7A6A5AA8: CRC32LookupTable(140) = &HE40ECF0B: CRC32LookupTable(141) = &H9309FF9D: CRC32LookupTable(142) = &HA00AE27: CRC32LookupTable(143) = &H7D079EB1
    CRC32LookupTable(144) = &HF00F9344: CRC32LookupTable(145) = &H8708A3D2: CRC32LookupTable(146) = &H1E01F268: CRC32LookupTable(147) = &H6906C2FE: CRC32LookupTable(148) = &HF762575D: CRC32LookupTable(149) = &H806567CB: CRC32LookupTable(150) = &H196C3671: CRC32LookupTable(151) = &H6E6B06E7
    CRC32LookupTable(152) = &HFED41B76: CRC32LookupTable(153) = &H89D32BE0: CRC32LookupTable(154) = &H10DA7A5A: CRC32LookupTable(155) = &H67DD4ACC: CRC32LookupTable(156) = &HF9B9DF6F: CRC32LookupTable(157) = &H8EBEEFF9: CRC32LookupTable(158) = &H17B7BE43: CRC32LookupTable(159) = &H60B08ED5
    CRC32LookupTable(160) = &HD6D6A3E8: CRC32LookupTable(161) = &HA1D1937E: CRC32LookupTable(162) = &H38D8C2C4: CRC32LookupTable(163) = &H4FDFF252: CRC32LookupTable(164) = &HD1BB67F1: CRC32LookupTable(165) = &HA6BC5767: CRC32LookupTable(166) = &H3FB506DD: CRC32LookupTable(167) = &H48B2364B
    CRC32LookupTable(168) = &HD80D2BDA: CRC32LookupTable(169) = &HAF0A1B4C: CRC32LookupTable(170) = &H36034AF6: CRC32LookupTable(171) = &H41047A60: CRC32LookupTable(172) = &HDF60EFC3: CRC32LookupTable(173) = &HA867DF55: CRC32LookupTable(174) = &H316E8EEF: CRC32LookupTable(175) = &H4669BE79
    CRC32LookupTable(176) = &HCB61B38C: CRC32LookupTable(177) = &HBC66831A: CRC32LookupTable(178) = &H256FD2A0: CRC32LookupTable(179) = &H5268E236: CRC32LookupTable(180) = &HCC0C7795: CRC32LookupTable(181) = &HBB0B4703: CRC32LookupTable(182) = &H220216B9: CRC32LookupTable(183) = &H5505262F
    CRC32LookupTable(184) = &HC5BA3BBE: CRC32LookupTable(185) = &HB2BD0B28: CRC32LookupTable(186) = &H2BB45A92: CRC32LookupTable(187) = &H5CB36A04: CRC32LookupTable(188) = &HC2D7FFA7: CRC32LookupTable(189) = &HB5D0CF31: CRC32LookupTable(190) = &H2CD99E8B: CRC32LookupTable(191) = &H5BDEAE1D
    CRC32LookupTable(192) = &H9B64C2B0: CRC32LookupTable(193) = &HEC63F226: CRC32LookupTable(194) = &H756AA39C: CRC32LookupTable(195) = &H26D930A: CRC32LookupTable(196) = &H9C0906A9: CRC32LookupTable(197) = &HEB0E363F: CRC32LookupTable(198) = &H72076785: CRC32LookupTable(199) = &H5005713
    CRC32LookupTable(200) = &H95BF4A82: CRC32LookupTable(201) = &HE2B87A14: CRC32LookupTable(202) = &H7BB12BAE: CRC32LookupTable(203) = &HCB61B38: CRC32LookupTable(204) = &H92D28E9B: CRC32LookupTable(205) = &HE5D5BE0D: CRC32LookupTable(206) = &H7CDCEFB7: CRC32LookupTable(207) = &HBDBDF21
    CRC32LookupTable(208) = &H86D3D2D4: CRC32LookupTable(209) = &HF1D4E242: CRC32LookupTable(210) = &H68DDB3F8: CRC32LookupTable(211) = &H1FDA836E: CRC32LookupTable(212) = &H81BE16CD: CRC32LookupTable(213) = &HF6B9265B: CRC32LookupTable(214) = &H6FB077E1: CRC32LookupTable(215) = &H18B74777
    CRC32LookupTable(216) = &H88085AE6: CRC32LookupTable(217) = &HFF0F6A70: CRC32LookupTable(218) = &H66063BCA: CRC32LookupTable(219) = &H11010B5C: CRC32LookupTable(220) = &H8F659EFF: CRC32LookupTable(221) = &HF862AE69: CRC32LookupTable(222) = &H616BFFD3: CRC32LookupTable(223) = &H166CCF45
    CRC32LookupTable(224) = &HA00AE278: CRC32LookupTable(225) = &HD70DD2EE: CRC32LookupTable(226) = &H4E048354: CRC32LookupTable(227) = &H3903B3C2: CRC32LookupTable(228) = &HA7672661: CRC32LookupTable(229) = &HD06016F7: CRC32LookupTable(230) = &H4969474D: CRC32LookupTable(231) = &H3E6E77DB
    CRC32LookupTable(232) = &HAED16A4A: CRC32LookupTable(233) = &HD9D65ADC: CRC32LookupTable(234) = &H40DF0B66: CRC32LookupTable(235) = &H37D83BF0: CRC32LookupTable(236) = &HA9BCAE53: CRC32LookupTable(237) = &HDEBB9EC5: CRC32LookupTable(238) = &H47B2CF7F: CRC32LookupTable(239) = &H30B5FFE9
    CRC32LookupTable(240) = &HBDBDF21C: CRC32LookupTable(241) = &HCABAC28A: CRC32LookupTable(242) = &H53B39330: CRC32LookupTable(243) = &H24B4A3A6: CRC32LookupTable(244) = &HBAD03605: CRC32LookupTable(245) = &HCDD70693: CRC32LookupTable(246) = &H54DE5729: CRC32LookupTable(247) = &H23D967BF
    CRC32LookupTable(248) = &HB3667A2E: CRC32LookupTable(249) = &HC4614AB8: CRC32LookupTable(250) = &H5D681B02: CRC32LookupTable(251) = &H2A6F2B94: CRC32LookupTable(252) = &HB40BBE37: CRC32LookupTable(253) = &HC30C8EA1: CRC32LookupTable(254) = &H5A05DF1B: CRC32LookupTable(255) = &H2D02EF8D
  End If

  CRC32Digest = StartingDigest
End Sub

Public Sub Process(ByRef Data() As Byte)
  'Processes Data
  '  Data [in] - zero-based byte array
  On Error Resume Next

  Dim tmpCount As Long
  tmpCount = UBound(Data()) + 1

  Dim i As Long
  For i = 0 To tmpCount - 1
    CRC32Digest = pShiftRightLong8(CRC32Digest) Xor CRC32LookupTable(CRC32Digest And &HFF Xor Data(i))
  Next i
End Sub

Public Function GetDigest() As Long
  'Returns CRC digest
  '  [out] - CRC digest
  GetDigest = CRC32Digest Xor &HFFFFFFFF
End Function


Public Function GetDigestFromData(ByRef Data() As Byte) As Long
  'Returns digest based on data array
  '  Data [in] - zero-based array of data to process
  Init
  Process Data()
  GetDigestFromData = GetDigest
End Function

Public Function GetDigestFromString(ByRef Data As String, Optional ByVal UseUnicode As Boolean = True) As Long
  'Returns digest based on given string
  '  Data [in] - string to process
  '  UseUnicode [in] - if false, data will be converted from unicode
  Init
  Dim tmpData() As Byte
  If UseUnicode = True Then
    tmpData() = Data
  Else
    tmpData() = StrConv(Data, vbFromUnicode)
  End If
  Process tmpData()
  GetDigestFromString = GetDigest
End Function


Private Sub Class_Initialize()
  Init
End Sub


Private Function pShiftRightLong8(ByVal Data As Long) As Long
  Dim tmpData As Long
  tmpData = (Data And &H7FFFFFFF) \ 256
  If (Data And &H80000000) <> 0 Then
    tmpData = tmpData Or &H800000
  End If
  pShiftRightLong8 = tmpData
End Function
