'Class that enables OpenFileDialog to work with large file count.
'
'Author
'  Name .....: Josip Medved
'  E-mail ...: support@jmedved.com
'  URL ......: http://www.jmedved.com?findbytitle=bigofd

'Portions are based on sample provided by Phil Johnson <philjohn@microsoft.com>

'Information
'  Additional : -
'  Requires ..: -
'  Has help ..: No
'  Has readme : No
'  Has sample : Yes
'  Last update: 2004-08-24 (yyyy-mm-dd)
'  Notes .....: -

'If you use this file in your program it would be nice to mention
'where you got it from, but you are not obligated to do so.

Option Compare Binary
Option Explicit On 
Option Strict On

Imports System
Imports Microsoft.VisualBasic
Imports System.Runtime.InteropServices

Public Class BigOFD

	Public Sub New()
		xBufferSize = 32767
	End Sub

	Public Sub New(ByVal multiSelect As Boolean)
		xMultiSelect = multiSelect
		xBufferSize = 32767
	End Sub

	Public Sub New(ByVal multiSelect As Boolean, ByVal bufferSize As Integer)
		xMultiSelect = multiSelect
		xBufferSize = bufferSize
	End Sub


	Private xAddExtension As Boolean = True
	Public Property AddExtension() As Boolean
		Get
			Return xAddExtension
		End Get
		Set(ByVal value As Boolean)
			xAddExtension = value
		End Set
	End Property

	Private xBufferSize As Integer = 32767
	Public Property BufferSize() As Integer
		Get
			Return xBufferSize
		End Get
		Set(ByVal value As Integer)
			If value >= 32767 Then xBufferSize = value
		End Set
	End Property

	Private xCheckFileExists As Boolean = True
	Public Property CheckFileExists() As Boolean
		Get
			Return xCheckFileExists
		End Get
		Set(ByVal value As Boolean)
			xCheckFileExists = value
		End Set
	End Property

	Private xCheckPathExists As Boolean = True
	Public Property CheckPathExists() As Boolean
		Get
			Return xCheckPathExists
		End Get
		Set(ByVal value As Boolean)
			xCheckPathExists = value
		End Set
	End Property

	Private xDefaultExt As String = ""
	Public Property DefaultExt() As String
		Get
			Return xDefaultExt
		End Get
		Set(ByVal value As String)
			xDefaultExt = value
		End Set
	End Property

	Private xDereferenceLinks As Boolean = True
	Public Property DereferenceLinks() As Boolean
		Get
			Return xDereferenceLinks
		End Get
		Set(ByVal value As Boolean)
			xDereferenceLinks = value
		End Set
	End Property

	Private xFileName As String
	Public Property FileName() As String
		Get
			Return xFileName
		End Get
		Set(ByVal value As String)
			xFileName = value
		End Set
	End Property

	Private xFileNames() As String
	Public Property FileNames() As String()
		Get
			Return xFileNames
		End Get
		Set(ByVal value As String())
			xFileNames = value
		End Set
	End Property

	Private xFilter As String = ""
	Public Property Filter() As String
		Get
			Return xFilter
		End Get
		Set(ByVal value As String)
			xFilter = value
		End Set
	End Property

	Private xFilterIndex As Integer = 1
	Public Property FilterIndex() As Integer
		Get
			Return xFilterIndex
		End Get
		Set(ByVal value As Integer)
			If value >= 1 Then FilterIndex = value
		End Set
	End Property

	Private xInitialDirectory As String = ""
	Public Property InitialDirectory() As String
		Get
			Return xInitialDirectory
		End Get
		Set(ByVal value As String)
			xInitialDirectory = value
		End Set
	End Property

	Private xMultiSelect As Boolean = False
	Public Property MultiSelect() As Boolean
		Get
			Return xMultiSelect
		End Get
		Set(ByVal value As Boolean)
			xMultiSelect = value
		End Set
	End Property

	Private xReadOnlyChecked As Boolean = False
	Public Property ReadOnlyChecked() As Boolean
		Get
			Return xReadOnlyChecked
		End Get
		Set(ByVal value As Boolean)
			xReadOnlyChecked = value
		End Set
	End Property

	Private xRestoreDirectory As Boolean = False
	Public Property RestoreDirectory() As Boolean
		Get
			Return xRestoreDirectory
		End Get
		Set(ByVal value As Boolean)
			xRestoreDirectory = value
		End Set
	End Property

	Private xShowHelp As Boolean = False
	Public Property ShowHelp() As Boolean
		Get
			Return xShowHelp
		End Get
		Set(ByVal value As Boolean)
			xShowHelp = value
		End Set
	End Property

	Private xShowReadOnly As Boolean = False
	Public Property ShowReadOnly() As Boolean
		Get
			Return xShowReadOnly
		End Get
		Set(ByVal value As Boolean)
			xShowReadOnly = value
		End Set
	End Property

	Private xTitle As String = ""
	Public Property Title() As String
		Get
			Return xTitle
		End Get
		Set(ByVal value As String)
			xTitle = value
		End Set
	End Property

	Private xValidateNames As Boolean = False
	Public Property ValidateNames() As Boolean
		Get
			Return xValidateNames
		End Get
		Set(ByVal value As Boolean)
			xValidateNames = value
		End Set
	End Property


	Public Overloads Function ShowDialog(ByVal owner As System.Windows.Forms.IWin32Window) As System.Windows.Forms.DialogResult
		Dim tmpOFN As New OpenFileName
		tmpOFN.structSize = Marshal.SizeOf(tmpOFN)
		tmpOFN.filter = xFilter.Replace("|", ChrW(0)) & ChrW(0)
		tmpOFN.filterIndex = xFilterIndex
		Dim tmpBuff As Integer = xBufferSize * Marshal.SystemDefaultCharSize
		tmpOFN.file = Marshal.AllocHGlobal(tmpBuff)
		LibWrap.RtlZeroMemory(tmpOFN.file, tmpBuff)
		tmpOFN.maxFile = xBufferSize - 2
		tmpOFN.fileTitle = New String(New Char(64) {})
		tmpOFN.maxFileTitle = tmpOFN.fileTitle.Length
		If owner Is Nothing Then
		Else
			tmpOFN.dlgOwner = owner.Handle
		End If

		tmpOFN.flags = OpenFileName.OFN_LONGNAMES Or OpenFileName.OFN_EXPLORER
		If (xCheckFileExists = True) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_FILEMUSTEXIST
		If (xCheckPathExists = True) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_PATHMUSTEXIST
		If (xAddExtension = True) AndAlso (xDefaultExt.Length > 0) Then tmpOFN.defExt = xDefaultExt
		If (xDereferenceLinks = False) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_NODEREFERENCELINKS
		If (xRestoreDirectory = True) AndAlso (xInitialDirectory.Length > 0) Then tmpOFN.initialDir = xInitialDirectory
		If (xMultiSelect = True) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_ALLOWMULTISELECT
		If (xReadOnlyChecked = True) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_READONLY
		If (xShowHelp = True) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_SHOWHELP
		If (xShowReadOnly = False) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_HIDEREADONLY Or tmpOFN.OFN_NOREADONLYRETURN
		If (xTitle.Length > 0) Then tmpOFN.title = xTitle
		If (xValidateNames = False) Then tmpOFN.flags = tmpOFN.flags Or tmpOFN.OFN_NOVALIDATE

		If LibWrap.GetOpenFileName(tmpOFN) Then
			Dim tmpFiles As String = Marshal.PtrToStringAuto(tmpOFN.file, xBufferSize)
			Marshal.FreeHGlobal(tmpOFN.file)
			tmpOFN.file = IntPtr.Zero

			If xMultiSelect = False Then
				Try
					xFileName = tmpFiles.Substring(0, tmpFiles.IndexOf(System.Convert.ToChar(0)))
					Erase xFileNames
					ReDim xFileNames(0)
					xFileNames(0) = xFileName
					If (xShowReadOnly = False) Then xReadOnlyChecked = ((tmpOFN.flags And tmpOFN.OFN_READONLY) = tmpOFN.OFN_READONLY)
				Catch ex As Exception
				End Try
			Else
				Try
					Dim iEndPos As Integer
					iEndPos = tmpFiles.IndexOf(New String(System.Convert.ToChar(0), 2))
					If iEndPos > 0 Then
						Dim tmpFileNames As String() = tmpFiles.Substring(0, iEndPos).Split(System.Convert.ToChar(0))
						Erase xFileNames
						If tmpFileNames.Length = 1 Then
							ReDim xFileNames(0)
							xFileNames(0) = tmpFileNames(0)
						Else
							If tmpFileNames(0).EndsWith("\") = False Then tmpFileNames(0) &= "\"
							ReDim xFileNames(UBound(tmpFileNames) - 1)
							For i As Integer = 0 To xFileNames.Length - 1
								xFileNames(i) = tmpFileNames(0) & tmpFileNames(i + 1)
							Next
						End If
						xFileName = xFileNames(0)
						If (xShowReadOnly = False) Then xReadOnlyChecked = ((tmpOFN.flags And tmpOFN.OFN_READONLY) = tmpOFN.OFN_READONLY)
					End If
				Catch ex As Exception
				End Try
			End If
			xInitialDirectory = tmpOFN.initialDir

			Return DialogResult.OK
			'Dim x As New System.Windows.Forms.OpenFileDialog
			'x.FileNames
		Else
			xFileName = ""
			Erase xFileNames
			Return DialogResult.Cancel
		End If
	End Function

	Public Overloads Function ShowDialog() As System.Windows.Forms.DialogResult
		Return ShowDialog(Nothing)
	End Function


	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> Private Class OpenFileName
		Public structSize As Integer = 0
		Public dlgOwner As IntPtr = IntPtr.Zero
		Public instance As IntPtr = IntPtr.Zero

		Public filter As String = Nothing
		Public customFilter As String = Nothing
		Public maxCustFilter As Integer = 0
		Public filterIndex As Integer = 0

		Public file As IntPtr
		Public maxFile As Integer = 0

		Public fileTitle As String = Nothing
		Public maxFileTitle As Integer = 0

		Public initialDir As String = Nothing

		Public title As String = Nothing

		Public flags As Integer = 0
		Public fileOffset As Short = 0
		Public fileExtension As Short = 0

		Public defExt As String = Nothing

		Public custData As IntPtr = IntPtr.Zero
		Public hook As IntPtr = IntPtr.Zero

		Public templateName As String = Nothing

		Public reservedPtr As IntPtr = IntPtr.Zero
		Public reservedInt As Integer = 0
		Public flagsEx As Integer = 0

		Public Const OFN_READONLY As Integer = &H1
		Public Const OFN_OVERWRITEPROMPT As Integer = &H2
		Public Const OFN_HIDEREADONLY As Integer = &H4
		Public Const OFN_NOCHANGEDIR As Integer = &H8
		Public Const OFN_SHOWHELP As Integer = &H10
		Public Const OFN_ENABLEHOOK As Integer = &H20
		Public Const OFN_ENABLETEMPLATE As Integer = &H40
		Public Const OFN_ENABLETEMPLATEHANDLE As Integer = &H80
		Public Const OFN_NOVALIDATE As Integer = &H100
		Public Const OFN_ALLOWMULTISELECT As Integer = &H200
		Public Const OFN_EXTENSIONDIFFERENT As Integer = &H400
		Public Const OFN_PATHMUSTEXIST As Integer = &H800
		Public Const OFN_FILEMUSTEXIST As Integer = &H1000
		Public Const OFN_CREATEPROMPT As Integer = &H2000
		Public Const OFN_SHAREAWARE As Integer = &H4000
		Public Const OFN_NOREADONLYRETURN As Integer = &H8000
		Public Const OFN_NOTESTFILECREATE As Integer = &H10000
		Public Const OFN_NONETWORKBUTTON As Integer = &H20000
		Public Const OFN_NOLONGNAMES As Integer = &H40000
		Public Const OFN_EXPLORER As Integer = &H80000
		Public Const OFN_NODEREFERENCELINKS As Integer = &H100000
		Public Const OFN_LONGNAMES As Integer = &H200000
	End Class

	Private Class LibWrap
		Declare Auto Function GetOpenFileName Lib "Comdlg32.dll" (<[In](), Out()> ByVal ofn As OpenFileName) As Boolean
		Declare Auto Sub RtlZeroMemory Lib "Kernel32.dll" (ByVal mem As IntPtr, ByVal size As Integer)
	End Class


	Private Shared Function UBound(ByVal array As String()) As Integer
		Try
			Return array.GetUpperBound(0)
		Catch ex As System.Exception
			Return -1
		End Try
	End Function

End Class
