'Class for performing unsigned bitwise calculations on signed integers.
'
'Author
'  Name .....: Josip Medved
'  E-mail ...: support@jmedved.com
'  URL ......: http://www.jmedved.com?findbyname=bitwise

'Information
'  Additional : -
'  Requires ..: -
'  Has help ..: No
'  Has readme : No
'  Has sample : Yes
'  Last update: 2004-10-28 (yyyy-mm-dd)
'  Notes .....: -

'If you use this file in your program it would be nice to mention
'where you got it from, but you are not obligated to do so.

Option Compare Binary
Option Explicit On 
Option Strict On

Public NotInheritable Class Bitwise
	Private Shared lookupMask16Pwr() As Short = {&H1, &H2, &H4, &H8, &H10, &H20, &H40, &H80, &H100, &H200, &H400, &H800, &H1000, &H2000, &H4000, &H8000S}
	Private Shared lookupMask16On() As Short = {&H0, &H1, &H3, &H7, &HF, &H1F, &H3F, &H7F, &HFF, &H1FF, &H3FF, &H7FF, &HFFF, &H1FFF, &H3FFF, &HBFFFS}
	Private Shared lookupMask32Pwr() As Integer = {&H1, &H2, &H4, &H8, &H10, &H20, &H40, &H80, &H100, &H200, &H400, &H800, &H1000, &H2000, &H4000, &H8000, &H10000, &H20000, &H40000, &H80000, &H100000, &H200000, &H400000, &H800000, &H1000000, &H2000000, &H4000000, &H8000000, &H10000000, &H20000000, &H40000000, &H80000000}
	Private Shared lookupMask32On() As Integer = {&H0, &H1, &H3, &H7, &HF, &H1F, &H3F, &H7F, &HFF, &H1FF, &H3FF, &H7FF, &HFFF, &H1FFF, &H3FFF, &H7FFF, &HFFFF, &H1FFFF, &H3FFFF, &H7FFFF, &HFFFFF, &H1FFFFF, &H3FFFFF, &H7FFFFF, &HFFFFFF, &H1FFFFFF, &H3FFFFFF, &H7FFFFFF, &HFFFFFFF, &H1FFFFFFF, &H3FFFFFFF, &HBFFFFFFF}	 ',&H0

#Region "ShiftLeft"

	Public Overloads Shared Function ShiftLeft(ByVal data As Integer, ByVal amount As Integer) As Integer
		If (amount <= 0) Then
			Return data
		ElseIf (amount > 31) Then
			Return 0
		Else
			If CBool(data And (lookupMask32Pwr(31 - amount))) Then
				Return ((data And lookupMask32On(31 - amount)) * (lookupMask32Pwr(amount))) Or &H80000000
			Else
				Return ((data And lookupMask32On(31 - amount + 1)) * (lookupMask32Pwr(amount)))
			End If
		End If
	End Function

	Public Overloads Shared Function ShiftLeft(ByVal data As Short, ByVal amount As Integer) As Short
		If (amount <= 0) Then
			Return data
		ElseIf (amount > 15) Then
			Return 0
		Else
			If CBool(data And (lookupMask16Pwr(15 - amount))) Then
				Return ((data And lookupMask16On(15 - amount)) * (lookupMask16Pwr(amount))) Or &H8000S
			Else
				Return ((data And lookupMask16On(15 - amount + 1)) * (lookupMask16Pwr(amount)))
			End If
		End If
	End Function

#End Region

#Region "ShiftRight"

	Public Overloads Shared Function ShiftRight(ByVal data As Integer, ByVal amount As Integer) As Integer
		If (amount <= 0) Then
			Return data
		ElseIf (amount > 31) Then
			Return 0
		Else
			If (data And lookupMask32Pwr(31)) = lookupMask32Pwr(31) Then
				Return (data And &H7FFFFFFF) \ lookupMask32Pwr(amount) Or lookupMask32Pwr(31 - amount)
			Else
				Return data \ lookupMask32Pwr(amount)
			End If
		End If
	End Function

	Public Overloads Shared Function ShiftRight(ByVal data As Short, ByVal amount As Integer) As Short
		If (amount <= 0) Then
			Return data
		ElseIf (amount > 15) Then
			Return 0
		Else
			If (data And lookupMask16Pwr(15)) = lookupMask16Pwr(15) Then
				Return (data And &H7FFFS) \ lookupMask16Pwr(amount) Or lookupMask16Pwr(15 - amount)
			Else
				Return data \ lookupMask16Pwr(amount)
			End If
		End If
	End Function

#End Region

#Region "RotateLeft"

	Public Overloads Shared Function RotateLeft(ByVal data As Integer, ByVal amount As Integer) As Integer
		Return ShiftLeft(data, amount) Or ShiftRight(data, 32 - amount)
	End Function

	Public Overloads Shared Function RotateLeft(ByVal data As Short, ByVal amount As Integer) As Short
		Return ShiftLeft(data, amount) Or ShiftRight(data, 16 - amount)
	End Function

#End Region

#Region "RotateRight"

	Public Overloads Shared Function RotateRight(ByVal data As Integer, ByVal amount As Integer) As Integer
		Return ShiftRight(data, amount) Or ShiftLeft(data, 32 - amount)
	End Function

	Public Overloads Shared Function RotateRight(ByVal data As Short, ByVal amount As Integer) As Short
		Return ShiftRight(data, amount) Or ShiftLeft(data, 16 - amount)
	End Function

#End Region

End Class
