'Class that enables serial port communication.
'
'Author
'  Name .....: Josip Medved
'  E-mail ...: support@jmedved.com
'  URL ......: http://www.jmedved.com?findbytitle=Com

'Information
'  Additional : -
'  Requires ..: -
'  Has help ..: No
'  Has readme : No
'  Has sample : Yes
'  Last update: 2004-12-10 (yyyy-mm-dd)
'  Notes .....: -

'If you use this file in your program it would be nice to mention
'where you got it from, but you are not obligated to do so.

Option Compare Binary
Option Explicit On 
Option Strict On

<System.CLSCompliant(False)> _
 Public Class Com

#Region "New"

	Public Sub New(ByVal port As Integer, ByVal baudRate As Integer, ByVal parity As ComParity, ByVal dataBits As Integer, ByVal stopBits As Integer, ByVal bufferSize As Integer, ByVal timeout As Integer)
		Me.Port = port
		Me.BaudRate = baudRate
		Me.Parity = parity
		Me.DataBits = dataBits
		Me.StopBits = stopBits
		Me.BufferSize = bufferSize
		Me.Timeout = timeout
	End Sub

	Public Sub New()
		Me.New(1, 9600, ComParity.None, 8, 1, 1024, 0)
	End Sub

	Public Sub New(ByVal port As Integer, ByVal baudRate As Integer)
		Me.New(port, baudRate, ComParity.None, 8, 1, 1024, 0)
	End Sub

	Public Sub New(ByVal settings As String)	 'COM1:9600,N,8,1
		Dim tmpPort As Integer = 1, tmpBaudRate As Integer = 9600, tmpParity As ComParity = ComParity.None, tmpDataBits As Integer = 8, tmpStopBits As Integer = 1
		Try
			Dim tmpSettingsA() As String = settings.Split(":"c)
			If (tmpSettingsA(0).Substring(0, 3).ToLower = "com") Then
				Dim tmpSettingsB() As String = tmpSettingsA(1).Split(","c)
				tmpPort = CInt(tmpSettingsA(0).Substring(3, tmpSettingsA(0).Length - 3))
				tmpBaudRate = CInt(tmpSettingsB(0))
				Select Case tmpSettingsB(1).Trim.ToLower
					Case "n" : tmpParity = ComParity.None
					Case "e" : tmpParity = ComParity.Even
					Case "o" : tmpParity = ComParity.Odd
					Case "m" : tmpParity = ComParity.Mark
				End Select
				tmpDataBits = CInt(tmpSettingsB(2))
				tmpStopBits = CInt(tmpSettingsB(3))
			End If
		Catch ex As Exception
		End Try
		Me.Port = tmpPort
		Me.BaudRate = tmpBaudRate
		Me.Parity = tmpParity
		Me.DataBits = tmpDataBits
		Me.StopBits = tmpStopBits
		Me.BufferSize = 1024
		Me.Timeout = 0
	End Sub

#End Region

	Private Const PURGE_RXCLEAR As Integer = &H8
	Private Const PURGE_TXCLEAR As Integer = &H4
	Private Const GENERIC_READ As Integer = &H80000000
	Private Const GENERIC_WRITE As Integer = &H40000000
	Private Const MS_CTS_ON As Integer = &H10
	Private Const MS_DSR_ON As Integer = &H20
	Private Const MS_RING_ON As Integer = &H40
	Private Const MS_RLSD_ON As Integer = &H80
	Private Const OPEN_EXISTING As Integer = 3
	Private Const SETRTS As Integer = 3
	Private Const CLRRTS As Integer = 4
	Private Const SETDTR As Integer = 5
	Private Const CLRDTR As Integer = 6
	Private Const SETBREAK As Integer = 8
	Private Const CLRBREAK As Integer = 9
	Private Const EV_RXCHAR As Integer = &H1
	Private Const EV_RXFLAG As Integer = &H2
	Private Const EV_TXEMPTY As Integer = &H4
	Private Const EV_CTS As Integer = &H8
	Private Const EV_DSR As Integer = &H10
	Private Const EV_RLSD As Integer = &H20
	Private Const EV_BREAK As Integer = &H40
	Private Const EV_ERR As Integer = &H80
	Private Const EV_RING As Integer = &H100
	Private Const EV_PERR As Integer = &H200
	Private Const EV_RX80FULL As Integer = &H400
	Private Const EV_EVENT1 As Integer = &H800
	Private Const EV_EVENT2 As Integer = &H1000

#Region "Win32API"

	<System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential, Pack:=1)> Private Structure DCB
		Public DCBlength As Integer
		Public BaudRate As Integer
		Public Bits1 As Integer
		Public wReserved As Short
		Public XonLim As Short
		Public XoffLim As Short
		Public ByteSize As Byte
		Public Parity As Byte
		Public StopBits As Byte
		Public XonChar As Byte
		Public XoffChar As Byte
		Public ErrorChar As Byte
		Public EofChar As Byte
		Public EvtChar As Byte
		Public wReserved2 As Short
	End Structure

	<System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential, Pack:=1)> Private Structure COMMTIMEOUTS
		Public ReadIntervalTimeout As Integer
		Public ReadTotalTimeoutMultiplier As Integer
		Public ReadTotalTimeoutConstant As Integer
		Public WriteTotalTimeoutMultiplier As Integer
		Public WriteTotalTimeoutConstant As Integer
	End Structure

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function BuildCommDCB(ByVal lpDef As String, ByRef lpDCB As DCB) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function ClearCommError(ByVal hFile As Integer, ByVal lpErrors As Integer, ByVal l As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function CloseHandle(ByVal hObject As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function CreateFile(<System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPStr)> ByVal lpFileName As String, ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, ByVal lpSecurityAttributes As Integer, ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function EscapeCommFunction(ByVal hFile As Integer, ByVal ifunc As Long) As Boolean
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function GetCommMask(ByVal hFile As Integer, ByRef lpEvtMask As Integer) As Boolean
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Public Shared Function GetCommModemStatus(ByVal hFile As Integer, ByRef lpModemStatus As Integer) As Boolean
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function GetCommState(ByVal hCommDev As Integer, ByRef lpDCB As DCB) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function GetLastError() As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function PurgeComm(ByVal hFile As Integer, ByVal dwFlags As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function ReadFile(ByVal hFile As Integer, ByVal Buffer As Byte(), ByVal nNumberOfBytesToRead As Integer, ByRef lpNumberOfBytesRead As Integer, ByVal lpOverlapped As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function SetCommMask(ByVal hFile As Integer, ByVal dwEvtMask As Integer) As Boolean
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function SetCommTimeouts(ByVal hFile As Integer, ByRef lpCommTimeouts As COMMTIMEOUTS) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function SetCommState(ByVal hCommDev As Integer, ByRef lpDCB As DCB) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function SetupComm(ByVal hFile As Integer, ByVal dwInQueue As Integer, ByVal dwOutQueue As Integer) As Integer
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function WaitCommEvent(ByVal hFile As Integer, ByRef lpEvtMask As Integer, ByVal lpOverlapped As Integer) As Boolean
	End Function

	<System.Runtime.InteropServices.DllImport("kernel32.dll")> Private Shared Function WriteFile(ByVal hFile As Integer, ByVal Buffer As Byte(), ByVal nNumberOfBytesToWrite As Integer, ByRef lpNumberOfBytesWritten As Integer, ByVal lpOverlapped As Integer) As Integer
	End Function

#End Region

	Private xHandle As Integer = -1

	Private xBaudRate As Integer = 9600
	<System.ComponentModel.DefaultValue(9600)> _
	Public Property BaudRate() As Integer
		Get
			Return xBaudRate
		End Get
		Set(ByVal value As Integer)
			xBaudRate = value
		End Set
	End Property

	Private xBufferSize As Integer = 1024
	<System.ComponentModel.DefaultValue(1024)> _
	Public Property BufferSize() As Integer
		Get
			Return xBufferSize
		End Get
		Set(ByVal value As Integer)
			xBufferSize = value
		End Set
	End Property

	Private xDataBits As Integer = 8
	<System.ComponentModel.DefaultValue(8)> _
	Public Property DataBits() As Integer
		Get
			Return xDataBits
		End Get
		Set(ByVal value As Integer)
			xDataBits = value
		End Set
	End Property

	Public WriteOnly Property DTREnable() As Boolean
		Set(ByVal value As Boolean)
			If xHandle <> -1 Then
				If value = True Then
					EscapeCommFunction(xHandle, SETDTR)
				Else
					EscapeCommFunction(xHandle, CLRDTR)
				End If
			End If
		End Set
	End Property

	Public WriteOnly Property RTSEnable() As Boolean
		Set(ByVal value As Boolean)
			If xHandle <> -1 Then
				If value Then
					EscapeCommFunction(xHandle, SETRTS)
				Else
					EscapeCommFunction(xHandle, CLRRTS)
				End If
			End If
		End Set
	End Property

	Public WriteOnly Property BREAKEnable() As Boolean
		Set(ByVal value As Boolean)
			If xHandle <> -1 Then
				If value Then
					EscapeCommFunction(xHandle, SETBREAK)
				Else
					EscapeCommFunction(xHandle, CLRBREAK)
				End If
			End If
		End Set
	End Property

	Public ReadOnly Property CDHolding() As Boolean
		Get
			If xHandle <> -1 Then
				Dim lpModemStatus As Integer
				If GetCommModemStatus(xHandle, lpModemStatus) Then
					Dim tmpState As Boolean = CBool(lpModemStatus And MS_RLSD_ON)
					lpModemStatus = Nothing
					Return tmpState
				End If
			End If
		End Get
	End Property

	Public ReadOnly Property CTSHolding() As Boolean
		Get
			If xHandle <> -1 Then
				Dim lpModemStatus As Integer
				If GetCommModemStatus(xHandle, lpModemStatus) Then
					Dim tmpState As Boolean = CBool(lpModemStatus And MS_CTS_ON)
					lpModemStatus = Nothing
					Return tmpState
				End If
			End If
		End Get
	End Property

	Public ReadOnly Property DSRHolding() As Boolean
		Get
			If xHandle <> -1 Then
				Dim lpModemStatus As Integer
				If GetCommModemStatus(xHandle, lpModemStatus) Then
					Dim tmpState As Boolean = CBool(lpModemStatus And MS_DSR_ON)
					lpModemStatus = Nothing
					Return tmpState
				End If
			End If
		End Get
	End Property

	Public ReadOnly Property RINGHolding() As Boolean
		Get
			If xHandle <> -1 Then
				Dim lpModemStatus As Integer
				If GetCommModemStatus(xHandle, lpModemStatus) Then
					Dim tmpState As Boolean = CBool(lpModemStatus And MS_RING_ON)
					lpModemStatus = Nothing
					Return tmpState
				End If
			End If
		End Get
	End Property

	Public ReadOnly Property IsOpen() As Boolean
		Get
			Return CBool(xHandle <> -1)
		End Get
	End Property

	Private xParity As ComParity = ComParity.None
	<System.ComponentModel.DefaultValue(ComParity.None)> _
	Public Property Parity() As ComParity
		Get
			Return xParity
		End Get
		Set(ByVal value As ComParity)
			xParity = value
		End Set
	End Property

	Private xPort As Integer = 1
	<System.ComponentModel.DefaultValue(1)> _
	Public Property Port() As Integer
		Get
			Return xPort
		End Get
		Set(ByVal Value As Integer)
			xPort = Value
		End Set
	End Property

	Private xStopBits As Integer = 1
	<System.ComponentModel.DefaultValue(1)> _
	Public Property StopBits() As Integer
		Get
			Return xStopBits
		End Get
		Set(ByVal Value As Integer)
			xStopBits = Value
		End Set
	End Property

	Private xTimeout As Integer = 0
	<System.ComponentModel.DefaultValue(0)> _
	Public Property Timeout() As Integer
		Get
			Return xTimeout
		End Get
		Set(ByVal value As Integer)
			If value <= 0 Then xTimeout = 50 Else xTimeout = value
			Dim uCtm As COMMTIMEOUTS
			If xHandle <> -1 Then
				With uCtm
					.ReadIntervalTimeout = 0
					.ReadTotalTimeoutMultiplier = 0
					.ReadTotalTimeoutConstant = xTimeout
					.WriteTotalTimeoutMultiplier = 0
					.WriteTotalTimeoutConstant = xTimeout
				End With
				SetCommTimeouts(xHandle, uCtm)
				uCtm = Nothing
			End If
		End Set
	End Property

	Public Sub PurgeComm(Optional ByVal rx As Boolean = True, Optional ByVal tx As Boolean = True)
		If xHandle <> -1 Then
			If rx = True Then PurgeComm(xHandle, PURGE_RXCLEAR)
			If tx = True Then PurgeComm(xHandle, PURGE_TXCLEAR)
		End If
	End Sub

	Public Sub Close()
		If xHandle <> -1 Then
			CloseHandle(xHandle)
			xHandle = -1
		End If
	End Sub

	Public Sub Open()
		Dim uDcb As DCB, tmpRet As Integer
		If xPort > 0 Then
			Try
				xHandle = CreateFile("COM" & xPort.ToString, GENERIC_READ Or GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0)
				If xHandle <> -1 Then
					Dim lpErrCode As Integer
					tmpRet = ClearCommError(xHandle, lpErrCode, 0)
					tmpRet = PurgeComm(xHandle, PURGE_RXCLEAR Or PURGE_TXCLEAR)
					tmpRet = GetCommState(xHandle, uDcb)
					Dim sParity As String = "NOEM".Substring(xParity, 1)
					Dim sDCBState As String = String.Format("baud={0} parity={1} data={2} stop={3}", xBaudRate, sParity, xDataBits, xStopBits)
					tmpRet = BuildCommDCB(sDCBState, uDcb)
					tmpRet = SetCommState(xHandle, uDcb)
					uDcb = Nothing
					sDCBState = Nothing
					tmpRet = SetupComm(xHandle, xBufferSize, xBufferSize)
					Timeout = xTimeout
				End If
			Catch ex As System.Exception
			End Try
		End If
	End Sub

	Public Overloads Function ReadString() As String
		Try
			Return System.Text.ASCIIEncoding.ASCII.GetString(Read())
		Catch ex As Exception
			Return ""
		End Try
	End Function

	Public Overloads Function ReadString(ByRef byteCount As Integer) As String
		Try
			Return System.Text.ASCIIEncoding.ASCII.GetString(Read(byteCount))
		Catch ex As Exception
			Return ""
		End Try
	End Function

	Public Overloads Function ReadString(ByRef byteCount As Integer, ByVal waitEvent As ComEvents) As String
		Try
			Return System.Text.ASCIIEncoding.ASCII.GetString(Read(byteCount, waitEvent))
		Catch ex As Exception
			Return ""
		End Try
	End Function

	Public Overloads Function Read(ByRef byteCount As Integer, ByVal waitEvent As ComEvents) As Byte()
		If (xHandle <> -1) Then
			If waitEvent <> ComEvents.None Then
				Dim tmpCommMask As Integer
				GetCommMask(xHandle, tmpCommMask)
				If CBool(CInt(tmpCommMask) And CInt(waitEvent)) = False Then
					SetCommMask(xHandle, CInt(waitEvent))
				End If
				WaitCommEvent(xHandle, tmpCommMask, Nothing)
			End If

			Try
				Dim tmpRet As Integer, tmpBuff(xBufferSize - 1) As Byte
				tmpRet = ReadFile(xHandle, tmpBuff, tmpBuff.Length, byteCount, 0)
				If tmpRet = 0 Then
				Else
					If byteCount > 0 Then
						Dim tmpBytes() As Byte
						ReDim tmpBytes(byteCount - 1)
						Array.Copy(tmpBuff, tmpBytes, byteCount)
						tmpBuff = Nothing
						Return tmpBytes
					End If
				End If
			Catch ex As System.Exception
			End Try
		End If
	End Function

	Public Overloads Function Read() As Byte()
		Return Read(0, ComEvents.None)
	End Function

	Public Overloads Function Read(ByRef byteCount As Integer) As Byte()
		Return Read(byteCount, ComEvents.None)
	End Function

	Public Overloads Sub Write(ByVal data As Byte(), Optional ByRef byteCount As Integer = 0)
		If data.Length = 0 Then Exit Sub

		If xHandle <> -1 Then
			Dim tmpRet As Integer
			Try
				Dim tmpData() As Byte
				tmpData = data
				PurgeComm(xHandle, PURGE_RXCLEAR Or PURGE_TXCLEAR)
				tmpRet = WriteFile(xHandle, tmpData, tmpData.Length, byteCount, 0)
				tmpData = Nothing
			Catch ex As System.Exception
			End Try
		End If
	End Sub

	Public Overloads Sub Write(ByVal data As String, Optional ByRef byteCount As Integer = 0)
		Try
			Write(System.Text.ASCIIEncoding.ASCII.GetBytes(data), byteCount)
		Catch ex As System.Exception
		End Try
	End Sub

End Class


#Region "Enums"

Public Enum ComParity As Integer
	None = 0
	Odd = 1
	Even = 2
	Mark = 3
End Enum

<FlagsAttribute()> _
Public Enum ComEvents As Integer
	None = 0
	RxChar = &H1
	RxFlag = &H2
	TxEmpty = &H4
	CTS = &H8
	DSR = &H10
	RLSD = &H20
	Break = &H40
	[Error] = &H80
	Ring = &H100
	PrinterError = &H200
	Rx80Full = &H400
	Event1 = &H800
	Event2 = &H1000
	Any = &H1000 Or &H800 Or &H400 Or &H200 Or &H100 Or &H80 Or &H40 Or &H20 Or &H10 Or &H8 Or &H4 Or &H2 Or &H1
End Enum

#End Region
