'Last update: 2005-11-27
'Created ...: 2004-11-13
'Author ....: Josip Medved
'E-mail ....: jmedved@jmedved.com
'Web .......: http://www.jmedved.com
Option Compare Binary
Option Explicit On
Option Strict On

Imports System.Runtime.InteropServices

''' <summary>
''' Class for handling named pipes operations.
''' </summary>
''' <remarks>
''' Based on http://support.microsoft.com/?kbid=871044.
''' </remarks>
Friend Class NamedPipe

	Friend Const BUFFER_INPUT As Integer = 16384
	Friend Const BUFFER_OUTPUT As Integer = 16384

	Private Class NativeMethods

		Friend Const FILE_FLAG_WRITE_THROUGH As Integer = &H80000000
		Friend Const FILE_SHARE_READ As Integer = &H1
		Friend Const FILE_SHARE_WRITE As Integer = &H2
		Friend Const GENERIC_READ As Integer = &H80000000
		Friend Const GENERIC_WRITE As Integer = &H40000000
		Friend Const INVALID_HANDLE_VALUE As Integer = -1
		Friend Const NMPWAIT_WAIT_FOREVER As Integer = &HFFFF
		Friend Const OPEN_EXISTING As Integer = 3
		Friend Const PIPE_ACCESS_DUPLEX As Integer = &H3
		Friend Const PIPE_READMODE_BYTE As Integer = &H0
		Friend Const PIPE_TYPE_BYTE As Integer = &H0
		Friend Const PIPE_UNLIMITED_INSTANCES As Integer = 255
		Friend Const PIPE_WAIT As Integer = &H0

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function CloseHandle(ByVal hObject As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=System.Runtime.InteropServices.CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function ConnectNamedPipe(ByVal hNamedPipe As Integer, ByVal lpOverlapped As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function CreateFile(ByVal lpFileName As String, ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, ByVal lpSecurityAttributes As Integer, ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function CreateNamedPipe(ByVal lpName As String, ByVal dwOpenMode As Integer, ByVal dwPipeMode As Integer, ByVal nMaxInstances As Integer, ByVal nOutBufferSize As Integer, ByVal nInBufferSize As Integer, ByVal nDefaultTimeOut As Integer, ByVal lpSecurityAttributes As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function DisconnectNamedPipe(ByVal hNamedPipe As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function ReadFile(ByVal hFile As Integer, ByRef lpBuffer As Byte, ByVal nNumberOfBytesToRead As Integer, ByRef lpNumberOfBytesRead As Integer, ByVal lpOverlapped As Integer) As Integer
		End Function

		<DllImport("kernel32.dll", CallingConvention:=CallingConvention.StdCall, CharSet:=CharSet.Unicode, SetLastError:=True)> _
		Friend Shared Function WriteFile(ByVal hFile As Integer, ByRef lpBuffer As Byte, ByVal nNumberOfBytesToWrite As Integer, ByRef lpNumberOfBytesWritten As Integer, ByVal lpOverlapped As Integer) As Integer
		End Function

	End Class

	''' <summary>
	''' Initializes object.
	''' </summary>
	''' <param name="pipeName">Pipe name. Must be up to 246 characters long and without backslashes (\).</param>
	''' <exception cref="System.ArgumentException">Pipe name must be between 1 and 247 characters long and it cannot contain backslash (\).</exception>
	Public Sub New(ByVal pipeName As String)
		Me.Name = pipeName
	End Sub

	Private _Name As String
	''' <summary>
	''' Returns pipe name.
	''' </summary>
	''' <value>Pipe name. Must be up to 246 characters long and without backslashes (\).</value>
	''' <returns>Pipe name.</returns>
	''' <exception cref="System.ArgumentException">Pipe name must be between 1 and 247 characters long and it cannot contain backslash (\).</exception>
	Public Property Name() As String
		Get
			Return _Name
		End Get
		Private Set(ByVal value As String)
			If Not ((value.Length >= 1) AndAlso (value.Length < 246)) Then Throw New ArgumentException("Pipe name must be between 1 and 247 characters long.")
			If (value.Contains("\")) Then Throw New ArgumentException("Pipe name cannot contain backslash (\).")
			_Name = value
		End Set
	End Property

	Public ReadOnly Property FullName() As String
		Get
			Return "\\.\pipe\" + Me.Name
		End Get
	End Property

	Private _Handle As Integer = NativeMethods.INVALID_HANDLE_VALUE
	''' <summary>
	''' Returns pipe handle.
	''' </summary>
	''' <value>Pipe handle.</value>
	''' <returns>Pipe handle.</returns>
	Public Property Handle() As Integer
		Get
			Return _Handle
		End Get
		Private Set(ByVal value As Integer)
			_Handle = value
		End Set
	End Property

	''' <summary>
	''' Creates named pipe.
	''' </summary>
	''' <exception cref="System.InvalidOperationException">CreateNamedPipeA returned invalid handle.</exception>
	Public Sub Create()
		Handle = NativeMethods.CreateNamedPipe(Me.FullName, NativeMethods.PIPE_ACCESS_DUPLEX Or NativeMethods.FILE_FLAG_WRITE_THROUGH, NativeMethods.PIPE_WAIT Or NativeMethods.PIPE_TYPE_BYTE Or NativeMethods.PIPE_READMODE_BYTE, NativeMethods.PIPE_UNLIMITED_INSTANCES, BUFFER_OUTPUT, BUFFER_INPUT, NativeMethods.NMPWAIT_WAIT_FOREVER, 0)
		If Handle = NativeMethods.INVALID_HANDLE_VALUE Then Throw New System.InvalidOperationException("CreateNamedPipeA returned invalid handle.")
	End Sub

	''' <summary>
	''' Opens existing named pipe.
	''' </summary>
	''' <exception cref="System.InvalidOperationException">CreateFile failed.</exception>
	Public Sub OpenExisting()
		Handle = NativeMethods.CreateFile(Me.FullName, NativeMethods.GENERIC_READ Or NativeMethods.GENERIC_WRITE, NativeMethods.FILE_SHARE_READ Or NativeMethods.FILE_SHARE_WRITE, 0, NativeMethods.OPEN_EXISTING, 0, 0)
		If Handle = NativeMethods.INVALID_HANDLE_VALUE Then Throw New System.InvalidOperationException("CreateFile returned invalid handle.")
	End Sub

	''' <summary>
	''' Closes handle.
	''' </summary>
	''' <exception cref="System.InvalidOperationException">CloseHandle failed.</exception>
	Public Sub Close()
		Dim result As Integer = NativeMethods.CloseHandle(Me.Handle)
		If result = 0 Then Throw New InvalidOperationException("CloseHandle failed.")
		Me.Handle = NativeMethods.INVALID_HANDLE_VALUE
	End Sub

	''' <summary>
	''' Connects to named pipe. Used as server side.
	''' </summary>
	''' <exception cref="System.InvalidOperationException">ConnectNamedPipe failed.</exception>
	Public Sub Connect()
		Dim result As Integer = NativeMethods.ConnectNamedPipe(Me.Handle, 0)
		If result = 0 Then Throw New InvalidOperationException("ConnectNamedPipe failed.")
	End Sub

	''' <summary>
	''' Disconnects named pipe. Used as server side.
	''' </summary>
	''' <exception cref="System.InvalidOperationException">DisconnectNamedPipe failed.</exception>
	Public Sub Disconnect()
		Dim result As Integer = NativeMethods.DisconnectNamedPipe(Me.Handle)
		If result = 0 Then Throw New InvalidOperationException("DisconnectNamedPipe failed.")
	End Sub

	''' <summary>
	''' Reads from defined pipe.
	''' </summary>
	''' <returns>Bytes read.</returns>
	''' <exception cref="System.InvalidOperationException">ReadFile failed.</exception>
	Public Function Read() As Byte()
		Dim buffer(0 To BUFFER_INPUT - 1) As Byte, refBufferLength As Integer = 0
		Dim result2 As Integer = NativeMethods.ReadFile(Me.Handle, buffer(0), buffer.Length, refBufferLength, 0)
		If result2 = 0 Then Throw New InvalidOperationException("ReadFile failed.")
		If refBufferLength = 0 Then
			Return New Byte() {}
		Else
			Dim retBuffer(0 To refBufferLength - 1) As Byte
			Array.Copy(buffer, retBuffer, refBufferLength)
			Return retBuffer
		End If
	End Function

	''' <summary>
	''' Writes data to defined pipe.
	''' </summary>
	''' <param name="value">Bytes to write.</param>
	''' <exception cref="System.InvalidOperationException">WriteFile failed.</exception>
	Public Sub Write(ByVal value As Byte())
		If value.Length > 0 Then
			Dim result As Integer = NativeMethods.WriteFile(Me.Handle, value(0), value.Length, 0, 0)
			If result = 0 Then Throw New InvalidOperationException("WriteFile failed.")
		End If
	End Sub

End Class
